/**
 * @(#) WASExpress6_0fixpackLnxMain.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set 
 * forth in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.wasexpress6_0fixpack_lnx;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.io.File;

/**
 * This class perform following tasks 
 * 1. Get the list of profiles. Stop default server server1 for each profile. 
 * 2. Run the backupconfig command to back up the configuration files. 
 * 3. Check if WAS_Install_Dir/updateinstaller/maintenance dir is there, if so rename it to WAS_Install_Dir/updateinstaller/maintenance_ERBackup. 
 * 4. untar or unzip package from unpacked dir to WAS_Install_Dir. 
 * 5. Run WAS_Install_Dir/bin/setupCmdLine.sh, WAS_Install_Dir/updateinstaller/update.sh -silent -W relaunch.active=false, and update -silent. 
 * 6. Verify install - A. Verify install success string in the update.log in the WAS_Install_Dir/logs/install and B. Run IVT.
 */
public class WASExpress6_0fixpackLnxMain extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2006 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
  private SupportLinuxHelper ivHelper;
  private String ivWasUsername = "";
  private String ivWasPassword = "";
  public static final String WAS_AUTHENTICATION_ERROR = "ADMN0022E";
  public static final String WAS_SEARCH_STRING_FOR_STOPPED = "ADMU0509I";
  public static final String WAS_SEARCH_STRING_FOR_SUCCESS = "ADMU4000I";
  private String ivInstallDir = null;
  private String ivTarFileName = null;

  /**
   *  Initialize variables
   */
  public int init(String[] args)
  {
    int returnCode = SUCCESS;
    ivHelper = getLinuxHelper();
    // set the message bundle and jar file for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");
    
    // Ensure the right number of parameters was passed
    if (1 == args.length)
    {
      ivTarFileName = args[0].trim();
      setVariableName("installDir");
      ivInstallDir = ivHelper.getIbmNsiPropValue(this);
      setVariableName("wasAdminUserName");
      ivWasUsername = ivHelper.getIbmNsiPropValue(this);
      setVariableName("wasAdminPassword");
      ivWasPassword = ivHelper.getIbmNsiPropValue(this);
    }
    else
    {
      String[] replacementText = { "WASExpress6_0fixpackLnxMain", "1", String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * Use the WebSphere stopServer command to validate the 
   * user id and password specified.  If the application server
   * fails to stop because of problems or authentication
   * issue a error message.
   * @return FAILURE if any of the application servers failed to stop.
   *   SUCCESS if all application servers were already stopped or stopped successfully.
   */
  private int stopWASServers()
  {
    //if a userid is specified, then make sure a password is also there
    if (ivWasUsername != null && !ivWasUsername.equals("") && (ivWasPassword == null || ivWasPassword.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { ivWasUsername }));
      ivHelper.postErrorMessageToDeployerUI(this);
      return FAILURE;
    }
    //if password is specified, make sure user id is specified
    if (ivWasPassword != null && !ivWasPassword.equals("") && (ivWasUsername == null || ivWasUsername.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { "\"\"" }));
      ivHelper.postErrorMessageToDeployerUI(this);
      return FAILURE;
    }

    //  get a list of profiles
    setPath(ivInstallDir);
    //String wasInstallDir = ivHelper.getdowsShortPath(this).replace('\\', '/').trim();
    String command = ivInstallDir + sSLASH + "bin" + sSLASH + "wasprofile.sh -listProfiles";

    //run the command and get the system command output
    String output = runCommand(command, command);
    setMessage(output);
    ivHelper.log(this);
    int rc = ivHelper.getLastCommandReturnCode(this);
    if (FAILURE == rc)
    {
      return FAILURE;
    }

    //Output from command is in the form of "[profile1, profile2, ...]<newline>"
    output = output.substring(output.indexOf("[") + 1, output.indexOf("]"));
    String[] profiles = output.split(",");

    //      format the userid/password portion of the WebSphere commands
    String secureString = "";
    String secureMaskString = "";
    if (ivWasUsername != null && ivWasPassword != null && !ivWasUsername.equals("") && !ivWasPassword.equals(""))
    {
      secureString = " -username " + ivWasUsername + " -password " + ivWasPassword;
      secureMaskString = " -username " + ivWasUsername + " -password ******";
    }

    for (int i = 0; i < profiles.length; ++i)
    {
      String profile = profiles[i].trim();
      //stop the server - assume server1
      String baseCommand = ivInstallDir + SLASH + "bin" + SLASH + "stopServer.sh server1 -profileName " + profile;
      String runCommand = baseCommand + secureString;
      String logCommand = baseCommand + secureMaskString;

      // Get the system Command out put
      output = runCommand(runCommand, logCommand);
      setMessage(output);
      ivHelper.log(this);
      rc = ivHelper.getLastCommandReturnCode(this);

      //Cannot rely on return code, so check the output for success message key
      if (output.indexOf(WAS_SEARCH_STRING_FOR_SUCCESS) == -1)
      {
        //Find out why the stopServer command failed. Check to see if it failed because of an invalid user id and/or password
        if (output.indexOf(WAS_AUTHENTICATION_ERROR) > 0)
        {
          //Log a more specific error if a user id/password were supplied
          if (!secureString.equals(""))
          {
            String[] replacementText = { ivWasUsername };
            setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, replacementText));
          }
          else
          {
            setMessage(getResourceString(NLSKeys.STOP_WASEXPRESS_FAIL));
          }
          ivHelper.postErrorMessageToDeployerUI(this);
          return FAILURE;
        }
        //Check to see if it was already stopped.
        if (output.indexOf(WAS_SEARCH_STRING_FOR_STOPPED) > 0)
        {
          setMessage(getResourceString(NLSKeys.STOP_WASEXPRESS_SUCCESS));
          ivHelper.log(this);
        }
        else
        {
          //The command failed for some other reason
          setMessage(getResourceString(NLSKeys.STOP_WASEXPRESS_FAIL));
          ivHelper.postErrorMessageToDeployerUI(this);
          return FAILURE;
        }
      }
    }
    return SUCCESS;
  }

  /**
   * Bback up all profiles configuration.  This uses the WAS backupConfig command.
   * The back up file created will be in the <install_location>/profiles
   * directory and have the format of WebSphereConfig_<year>_<month>_<day>_<profileName>.zip.
   * The log file <install_location>/profiles/<year>_<month>_<day>_profileBackup.log is
   * also created containing the results of the WAS back up commands.
   * 
   * @return SUCCESS - if listProfiles and backupConfig command returns successfully 
   *         FAILURE otherwise.
   */
  private int backupConfiguration()
  {
    int returnCode = SUCCESS;

    String filePrefix = new SimpleDateFormat("yyyy_MM_dd").format(new Date()) + "_";
    //get a list of profiles
    setPath(ivInstallDir);
    String command = ivInstallDir + SLASH + "bin" + SLASH + "wasprofile.sh -listProfiles";
    String output = runCommand(command, command);
    setMessage(output);
    ivHelper.log(this);
    if (ivHelper.getLastCommandReturnCode(this) != 0)
    {
      returnCode = FAILURE;
    }
    if (returnCode != FAILURE)
    {
      //Call back up script for each profile. Output from command is in the form of "[profile1, profile2, ...]<newline>"
      output = output.substring(1, output.indexOf("]"));
      String[] profiles = output.split(",");

      //format the backup file and log file
      String profileDir = ivInstallDir + SLASH + "profiles" + SLASH;
      String backupLog = profileDir + filePrefix + "profileBackup.log";

      //format the userid/password portion of the WebSphere commands
      String secureString = "";
      String secureMaskString = "";
      if (ivWasUsername != null && ivWasPassword != null && !ivWasUsername.equals("") && !ivWasPassword.equals(""))
      {
        secureString = " -username " + ivWasUsername + " -password " + ivWasPassword;
        secureMaskString = " -username " + ivWasUsername + " -password ******";
      }
      for (int i = 0; i < profiles.length; ++i)
      {
        String profile = profiles[i].trim();
        //call the backupConfig command
        String baseCommand = ivInstallDir + SLASH + "bin" + SLASH + "backupConfig.sh " + profileDir
            + "WebSphereConfig_" + filePrefix + profile + ".zip" + " -logfile " + backupLog + " -profileName "
            + profile;
        String runCommand = baseCommand + secureString;
        String logCommand = baseCommand + secureMaskString;
        runCommand(runCommand, logCommand);
        if (ivHelper.getLastCommandReturnCode(this) != 0)
        {
          setMessage(getResourceString(NLSKeys.WAS_PROFILE_BACKUP_FAILED, new String[] { profile }));
          ivHelper.log(this);
          returnCode = FAILURE;
        }
      }
      //append backup log to our log
      setFileName(backupLog);
      if (ivHelper.fileExists(this))
      {
        ivHelper.logAppendFile(this);
      }
    }
    return returnCode;
  }

  /**
   * Rename update installer directory if already exist in the WAS install root. Renaming needs to be done because 
   * in order to use latest update installer provided with fix pack image we need to copy it to the WAS install root.
   */
  private void renameUpdateInstaller()
  {
    String dirSuffix = new SimpleDateFormat("yyyy_MM_dd").format(new Date());
    String updateInstallerLocation = ivInstallDir + SLASH + "updateinstaller";
    setFileName(updateInstallerLocation);
    if (ivHelper.fileExists(this))
    {
      //rename it
      File newFile = new File(updateInstallerLocation + dirSuffix);
      File renamefile = new File(updateInstallerLocation);
      renamefile.renameTo(newFile);
    }
  }

  /**
   * Unpack the product image tar file in the WAS install root directory
   * @param tarFile The tar file to be extracted
   * 
   * @return SUCCESS if successful extracted, FAILURE otherwise
   */
  private int performExtract(String fileName)
  {
    int returnCode = SUCCESS;
    //command to extract the image
    String[] command = { "/bin/sh", "-c", "tar xf " + ivHelper.getUnpackedDir(this) + fileName + " -C " + ivInstallDir };

    setMessage(getResourceString(NLSKeys.CMDINVOKED, command));
    ivHelper.log(this);

    //invoke the command
    setCommandArray(command);
    //log command
    setMessage(getResourceString(NLSKeys.CMDINVOKED, ivHelper.getStringFromCommandArray(this)));
    ivHelper.log(this);

    //run the command
    returnCode = ivHelper.launchAndWait(this);

    //create message to log command success or fail
    if (SUCCESS == returnCode)
    {
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
    }
    else
    {
      setMessage(getResourceString(NLSKeys.CMD_FAIL, String.valueOf(returnCode)));
    }
    //log messages
    ivHelper.log(this);
    
    return returnCode;
  }

  /**
   * Run update installer to install fix pack.
   * 
   * @return SUCCESS - if INSTCONFSUCCESS and INSTCONFPARTIALSUCCESS string found in the update installer log
   *         FAILURE otherwise.
   */
  private int Install()
  {
    int returnCode = FAILURE;
    if(performExtract(ivTarFileName) == returnCode)
    {
      return returnCode;
    }
    
    long logStartSize = 0;    
    String SUCCESS_SEARCH_STRING = "INSTCONFSUCCESS";
    String WARNING_SEARCH_STRING = "INSTCONFPARTIALSUCCESS";
    //the update log containing success or partial success or failulre message
    String logFile = ivInstallDir + SLASH + "logs" + SLASH + "update" + SLASH
        + "6.0.2-WS-WAS-LinuxX32-FP00000017.install" + SLASH + "updatelog.txt";
    //get the size to be used later for checking
    File log = new File(logFile);      
    logStartSize = log.length();
    
    //command to initialize server when it is reinstall
    String [] command = { "/bin/sh", "-c",
        "cd " + ivInstallDir + SLASH + "bin" + ";"  + "./setupCmdLine.sh" + ";" +
        "cd " + ivInstallDir + SLASH + "updateinstaller" + ";" +
        "./update -silent -W relaunch.active=false"};
    setCommandArray(command);

    //log command
    setMessage(getResourceString(NLSKeys.CMDINVOKED, ivHelper.getStringFromCommandArray(this)));
    ivHelper.log(this);

    //run the command
    returnCode = ivHelper.launchAndWait(this);

    //create message to log command success or fail
    if (SUCCESS == returnCode)
    {
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
    }
    else
    {
      setMessage(getResourceString(NLSKeys.CMD_FAIL, String.valueOf(returnCode)));
    }
    //log messages
    ivHelper.log(this);



    setFileName(logFile);
    if (ivHelper.fileExists(this))
    {
      File logFileFile = new File(logFile);
      // Append the install log
      ivHelper.logAppendFile(this);
      if (logFileFile.length() != logStartSize)
      {
        //search for INSTCONFSUCCESS success message
        setGrepFileName(logFile);
        setGrepString(SUCCESS_SEARCH_STRING);
        setFileTextStartIndex((new Long(logStartSize)).intValue());
        if (getHelper().fileGrep(this))
        {
          returnCode = SUCCESS;
        }
        //search for any INSTCONFPARTIALSUCCESS warning messages
        setGrepString(WARNING_SEARCH_STRING);
        if (ivHelper.fileGrep(this))
        {
          setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
          ivHelper.postWarningMessageToDeployerUI(this);
          returnCode = SUCCESS;
        }
      }
    }
    else
    {
      setMessage(getResourceString(NLSKeys.LOG_FILE_NOT_FOUND, new String[] { logFile }));
      ivHelper.log(this);
    }
    return returnCode;
  }

  /**
   * Run WebSphere Installation Verification Test (IVT) using a default profile "default" and default server "server1".
   * 
   * @return - return code of IVT command.
   */
  private int verifyInstall()
  {
    String profileBinDir = ivInstallDir + SLASH + "profiles" + SLASH + "default" + sSLASH + "bin";
    //IVT command
    String command = profileBinDir + SLASH + "ivt.sh server1 default";
    String output = runCommand(command, command);
    setMessage(output);
    ivHelper.log(this);
    //return the command return code
    return ivHelper.getLastCommandReturnCode(this);
  }

  /**Run a system command. Return the ouput from the command to the caller.
   * The logCommand string is logged along with the appropriate success or
   * failure message.  If return code is needed from the command, 
   * use the ivHelper.getLastCommandRerturnCode(this)method to retrieve it.
   * 
   * @param - comamnd to run and command to log.
   * @return The output from the command run.
   */
  private String runCommand(String inCommand, String logCommand)
  {
    setCommand(inCommand);
    setMessage(getResourceString(NLSKeys.CMDINVOKED, new String[] { logCommand }));
    ivHelper.log(this);
    //Run the command and capture the output
    String output = ivHelper.getSystemCommandOutput(this);
    //Log success or failure
    int rc = ivHelper.getLastCommandReturnCode(this);
    if (rc == 0)
    {
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
    }
    else
    {
      setMessage(getResourceString(NLSKeys.CMD_FAIL, new String[] { Integer.toString(rc) }));
    }
    ivHelper.log(this);
    return output;
  }

  /**
   * MAIN
   */
  public static void main(String[] args)
  {
    WASExpress6_0fixpackLnxMain main = new WASExpress6_0fixpackLnxMain();
    main.init(args);
    //stop servers if running
    int returnCode = main.stopWASServers();
    if (returnCode == SUCCESS)
    {
      //backup configuration files
      main.backupConfiguration();
    }
    if (returnCode == SUCCESS)
    {
      //rename updateinstaller directory if exist
      main.renameUpdateInstaller();
      //extract the image and install fix pack
      returnCode = main.Install();
    }
    if (returnCode == SUCCESS)
    {
      //verify install
      main.verifyInstall();
    }
    System.exit(returnCode);
  }
}